package com.fundi.framework.common.comparator;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;
import java.util.HashMap;
import java.util.Iterator;
import java.util.Map;
import java.util.Set;

import javax.xml.bind.annotation.XmlRootElement;

@XmlRootElement
public class ComparatorConfiguration {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$

	private static final long serialVersionUID = 1L;
	
	private String context = ""; //$NON-NLS-1$
	
	private ArrayList<String> keyFields = new ArrayList<String>();
	private ArrayList<String> ignoreFields = new ArrayList<String>();
	private ArrayList<String> hideFields = new ArrayList<String>();
	private boolean showAllKeys = true;
	private boolean showMatchedKeys = false;
	private boolean showUnmatchedKeys = false;
	
	private Map<String, String> fieldLabels = new HashMap<String, String>();

	public ComparatorConfiguration() {
		
		super();
	}
	
	public ComparatorConfiguration(String context) {
		
		super();
		
		this.context = context;
	}
	
	public void setConfiguration(ComparatorHelper helper, ArrayList<ComparatorField> fields) {
		
		showAllKeys = helper.showingAllKeys();
		showMatchedKeys = helper.showingMatchedKeys();
		showUnmatchedKeys = helper.showingUnmatchedKeys();
		
		/*
		 * When saving the labels for fields, just save the label for the field in the first data source, as it's assumed to be the primary source. 
		 */
		for (ComparatorField field : fields) {
			fieldLabels.put(field.getSourceFields().get(0), field.getLabel());
		}
		keyFields = helper.getKeyFields();
		ignoreFields = helper.getIgnoreFields();
		hideFields = helper.getHideFields();
	}

	public void restoreConfiguration(ComparatorHelper helper) {

		String[] labels = helper.getLabels();
		String[][] sourceFields = helper.getSourceFields();
		int idxFld;
		String label;
		
		helper.setShowAllKeys(showAllKeys);
		helper.setShowMatchedKeys(showMatchedKeys);
		helper.setShowUnmatchedKeys(showUnmatchedKeys);
		
		/*
		 * Where a field for the first data source matches a saved one, default its label to the saved label. 
		 */
		for (idxFld = 0; idxFld < sourceFields[0].length; idxFld++) {
			label = fieldLabels.get(sourceFields[0][idxFld]);
			if (label != null) {
				labels[idxFld] = label;
			}
		}
		
		helper.restoreKeyFields(keyFields);
		helper.restoreIgnoreFields(ignoreFields);
		helper.restoreHideFields(hideFields);
	}

	public String getContext() {
		return context;
	}

	public void setContext(String context) {
		this.context = context;
	}

	public ArrayList<String> getKeyFields() {
		return keyFields;
	}

	public void setKeyFields(ArrayList<String> keyFields) {
		this.keyFields = keyFields;
	}

	public ArrayList<String> getIgnoreFields() {
		return ignoreFields;
	}

	public void setIgnoreFields(ArrayList<String> ignoreFields) {
		this.ignoreFields = ignoreFields;
	}

	public ArrayList<String> getHideFields() {
		return hideFields;
	}

	public void setHideFields(ArrayList<String> hideFields) {
		this.hideFields = hideFields;
	}

	public boolean isShowAllKeys() {
		return showAllKeys;
	}

	public void setShowAllKeys(boolean showAllKeys) {
		this.showAllKeys = showAllKeys;
	}

	public boolean isShowMatchedKeys() {
		return showMatchedKeys;
	}

	public void setShowMatchedKeys(boolean showMatchedKeys) {
		this.showMatchedKeys = showMatchedKeys;
	}

	public boolean isShowUnmatchedKeys() {
		return showUnmatchedKeys;
	}

	public void setShowUnmatchedKeys(boolean showUnmatchedKeys) {
		this.showUnmatchedKeys = showUnmatchedKeys;
	}

	public Map<String, String> getFieldLabels() {
		return fieldLabels;
	}

	public void setFieldLabels(Map<String, String> fieldLabels) {
		this.fieldLabels = fieldLabels;
	}
}
