package com.fundi.framework.common.cslcomms;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.util.ArrayList;
import java.util.Iterator;
import java.util.regex.Matcher;
import java.util.regex.Pattern;

import com.fundi.framework.common.nl1.Messages;

public class CSLObject implements java.io.Serializable {
	
	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private ArrayList<CSLField> fields = new ArrayList<CSLField>();
	
	private ArrayList<CSLFieldDefn> defns;
	private String rsp;
	private boolean type2;
	private String cmd;
	
	private Object provider = null;  //The source of this object - a GPL Repository if via a GPL command, or an IMSplex if via a DRD command.
	
	private Pattern	parenPattern = Pattern.compile("[\\(\\)]"); //$NON-NLS-1$
	private static String START_PARENS = Messages.getString("CSLObject_4"); //$NON-NLS-1$
	private static char LEFT_PARENS = Messages.getString("CSLObject_5").charAt(0); //$NON-NLS-1$
	private static char RIGHT_PARENS = Messages.getString("CSLObject_5").charAt(0); //$NON-NLS-1$
	
	public CSLObject() {
		
		super();
	}
	
	public CSLObject(ArrayList<CSLFieldDefn>defns, String rsp, boolean type2, String cmd) {
		
		super();

		int fldStart = 0;
		int fldEnd = 0;
		int firstParens;
		int parenCnt = 0;
		Matcher	matcher;
		char 	parens;
		boolean valid = true;
		
		this.defns = defns;
		this.rsp = rsp;
		
		this.type2 = type2;
		this.cmd = cmd;
		
		if (type2) {
			//The rsp is a <rsp> object within a <cmdrspdata> object, but without the <rsp> and </rsp> tags.
			
			//Extract each field value.  They are in the format shortLabel(value), and are separated by spaces.
			//We assume the value may itself contain spaces or parentheses, but if it contains parentheses they must be matched.
			matcher = parenPattern.matcher(rsp);
				while (valid && matcher.find(fldEnd)) {
				fldEnd = matcher.end();
				parens = rsp.charAt(matcher.start());
				if (parens == LEFT_PARENS) {
					parenCnt++;
				} else {
					parenCnt--;
				}
				if (parenCnt < 0) {
					//invalid format, more ')' to this point than '('
					valid = false;
				}
				if (parenCnt == 0) {  
					//Found the boundaries of a field value.
					fields.add(new CSLField(defns, rsp.substring(fldStart, fldEnd)));
					for (fldStart = fldEnd; fldStart < rsp.length(); fldStart++) {
						if (!(rsp.charAt(fldStart) == ' ')) {  //Start of next field (doesn't assume single space between them). //$NON-NLS-1$
							break;
						}
					}
				}
			}
		} else {
			//The rsp is a <mbr> object within a <msgdata> object, but without the <mbr> and </mbr> tags.
			fldStart = rsp.indexOf(Messages.getString("CSLObject_8")); //$NON-NLS-1$
			if (fldStart >= 0) {
				fldEnd = rsp.indexOf(Messages.getString("CSLObject_9")); //$NON-NLS-1$
				fields.add(new CSLField(defns, Messages.getString("CSLObject_10"), rsp.substring(fldStart + 6, fldEnd))); //$NON-NLS-1$
			}
			fldStart = rsp.indexOf(Messages.getString("CSLObject_11")); //$NON-NLS-1$
			fields.add(new CSLField(defns, Messages.getString("CSLObject_12"), rsp.substring(fldStart + 5))); //$NON-NLS-1$
		}
	}
	
	public String getFieldValue(String slbl) {
		
		Iterator<CSLField> it;
		CSLField field;
		
		for (it = fields.iterator(); it.hasNext();) {
			field = it.next();
			if (field.getSlbl().equalsIgnoreCase(slbl)) {
				if (field.getValue() != null) {
					return field.getValue();
				}
				else {
					return ""; //$NON-NLS-1$
				}
			}
		}
		
		return ""; //$NON-NLS-1$
	}
	
	public String getFieldValueByLlbl(String llbl) {
		
		Iterator<CSLField> it;
		CSLField field;
		
		for (it = fields.iterator(); it.hasNext();) {
			field = it.next();
			if (field.getLlbl().equalsIgnoreCase(llbl)) {
				if (field.getValue() != null) {
					return field.getValue();
				}
				else {
					return ""; //$NON-NLS-1$
				}
			}
		}
		
		return ""; //$NON-NLS-1$
	}
	
	public int getIntFieldValue(String slbl) {
		
		return Integer.parseInt(getFieldValue(slbl));
	}
	
	public int getIntValueByLlbl(String llbl) {
		
		Iterator<CSLField> it;
		CSLField field;
		
		for (it = fields.iterator(); it.hasNext();) {
			field = it.next();
			if (field.getLlbl().equalsIgnoreCase(llbl)) {
				return Integer.parseInt(getFieldValueByLlbl(llbl));
			}
		}
		
		return 0;
	}
	
	public CSLField getKeyField() {
		
		Iterator<CSLField> it;
		CSLField field;
		
		for (it = fields.iterator(); it.hasNext();) {
			field = it.next();
			if (field.isKey()) {
				return field;
			}
		}
		
		return null;
	}
	
	public ArrayList<CSLField> getFields() {
		
		return fields;
	}
	
	public CSLField getFieldByLlbl(String lLbl) {
		
		Iterator<CSLField> it;
		CSLField field;
		
		for (it = fields.iterator(); it.hasNext();) {
			field = it.next();
			if (field.getLlbl().equalsIgnoreCase(lLbl)) {
				return field;
			}
		}
		
		return null;
	}
	
	public ArrayList<String> getFieldLlbls() {
		
		ArrayList<String> names = new ArrayList<String>();
		Iterator<CSLField> it;
		
		for (it = fields.iterator(); it.hasNext();) {
			names.add(it.next().getLlbl());
		}
		
		return names;
	}
	
	public ArrayList<Class> getFieldDataTypes() {
		
		ArrayList<Class> types = new ArrayList<Class>();
		Iterator<CSLField> it;
		String type;
		
		for (it = fields.iterator(); it.hasNext();) {
			type = it.next().getDataType();
			if (type.equals(CSLFieldDefn.DTYPE_CHAR)) {
				types.add(String.class);
			}
			else {
				types.add(int.class);
			}
		}
		
		return types;
	}
	
	public ArrayList<CSLFieldDefn> getDefns() {
		return defns;
	}

	public String getRsp() {
		return rsp;
	}

	public boolean isType2() {
		return type2;
	}

	public String getCmd() {
		return cmd;
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((fields == null) ? 0 : fields.hashCode());
		result = prime * result + (type2 ? 1231 : 1237);
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		CSLObject other = (CSLObject) obj;
		if (fields == null) {
			if (other.fields != null)
				return false;
		} else if (!fields.equals(other.fields))
			return false;
		if (type2 != other.type2)
			return false;
		return true;
	}

	public Object getProvider() {
		
		return provider;
	}

	public void setProvider(Object provider) {
		
		this.provider = provider;
	}
	
	public CSLField getFieldByKeyLabel(String keyLabel) {
		CSLField result;
		
		result = null;
		for (CSLField fld: fields) {
			if (fld.getKeyLabel().equals(keyLabel)) {
				result = fld;
				break;
			}
		}
		
		return result;
	}
	
	public String getFieldValueByKeyLabel(String keyLabel) {
		CSLField resultField;
		
		resultField = getFieldByKeyLabel(keyLabel);
		
		return (resultField == null ? null : resultField.getValue()); 
	}
}
