package com.fundi.framework.common.instance;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;

import com.fundi.framework.common.nl1.Messages;
import com.fundi.framework.common.util.Collator;

public class ConnectionProfile implements Serializable, Comparable {

	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;
	
	private String name = ""; //$NON-NLS-1$
	private String userid = ""; //$NON-NLS-1$
	private String password = ""; //$NON-NLS-1$
	private boolean defaultProfile = false;
	
	/*
	 *  These variables aren't used yet.  They were added on 3/5/2013 because we'll need to support SSL/TLS connections soon.
	 */
	private boolean usingSSL = false;
	private String sslProtocolHandler = "com.sun.net.ssl.internal.www.protocol"; //$NON-NLS-1$
	private String keyStore = "C:\\Program Files\\Java\\jre\\lib\\security\\???.jks"; //$NON-NLS-1$
	private String keyStorePswd = "password"; //$NON-NLS-1$
	private String trustStore = "C:\\Program Files\\Java\\jre\\lib\\security\\???.jks"; //$NON-NLS-1$
	private String trustStorePswd = "password"; //$NON-NLS-1$
	
	private boolean useable = true;  //Set to false if a connection attempt via this profile fails due to detected userid/password issues.
	private String unuseableReason = "";  //Detecting application should set a text reason which is displayed to the user.
	private boolean promptForDetailsEnabled = true;  //If false, Framework must not prompt for details if this profile is unuseable;
	
	public ConnectionProfile() {

		super();
	}
	
	public ConnectionProfile(ConnectionProfile profile) {
		
		//Create a deep copy of the original.
		
		setName(profile.getName());
		setUserid(profile.getUserid());
		setPassword(profile.getPassword());
		setDefaultProfile(profile.isDefaultProfile());
		setUsingSSL(profile.isUsingSSL());
		setSslProtocolHandler(profile.getSslProtocolHandler());
		setKeyStore(profile.getKeyStore());
		setKeyStorePswd(profile.getKeyStorePswd());
		setTrustStore(profile.getTrustStore());
		setTrustStorePswd(profile.getTrustStorePswd());
		if (profile.isUseable()) {
			setUseable(useable);
		}
		else {
			setUnuseable(profile.getUnuseableReason());
		}
	}
	
	public String getName() {
		return name;
	}
	
	public void setName(String name) {
		this.name = name;
	}
	
	public String getUserid() {
		return userid;
	}
	
	public void setUserid(String userid) {
		this.userid = userid;
	}
	
	public String getPassword() {
		return password;
	}
	
	public void setPassword(String password) {
		this.password = password;
	}
	
	public boolean isDefaultProfile() {
		return defaultProfile;
	}
	
	public void setDefaultProfile(boolean defaultProfile) {
		this.defaultProfile = defaultProfile;
	}
	
	@Override
	public int compareTo(Object arg0) {

		return (new Collator()).compare(getName(), ((ConnectionProfile) arg0).getName());
	}

	public boolean isUsingSSL() {
		return usingSSL;
	}

	public void setUsingSSL(boolean usingSSL) {
		this.usingSSL = usingSSL;
	}

	public String getSslProtocolHandler() {
		return sslProtocolHandler;
	}

	public void setSslProtocolHandler(String sslProtocolHandler) {
		this.sslProtocolHandler = sslProtocolHandler;
	}

	public String getKeyStore() {
		return keyStore;
	}

	public void setKeyStore(String keyStore) {
		this.keyStore = keyStore;
	}

	public String getKeyStorePswd() {
		return keyStorePswd;
	}

	public void setKeyStorePswd(String keyStorePswd) {
		this.keyStorePswd = keyStorePswd;
	}

	public String getTrustStore() {
		return trustStore;
	}

	public void setTrustStore(String trustStore) {
		this.trustStore = trustStore;
	}

	public String getTrustStorePswd() {
		return trustStorePswd;
	}

	public void setTrustStorePswd(String trustStorePswd) {
		this.trustStorePswd = trustStorePswd;
	}

	public boolean isUseable() {
		return useable;
	}

	public void setUseable(boolean useable) {
		this.useable = useable;
		unuseableReason = "";
		promptForDetailsEnabled = true;
	}

	public String getUnuseableReason() {
		return unuseableReason;
	}

	public void setUnuseable(String unuseableReason) {
		useable = false;
		this.unuseableReason = unuseableReason;
	}
	
	public boolean isPromptForDetailsEnabled() {
		return promptForDetailsEnabled;
	}

	public void setPromptForDetailsEnabled(boolean promptForDetailsEnabled) {
		this.promptForDetailsEnabled = promptForDetailsEnabled;
	}
}
