package com.fundi.framework.common.instance;

/*
 *  Common Services Library - Framework plug-in.
 *  (C) Copyright Fundi Software 2011, 2016.  Fundi Confidential.
 *  All rights reserved.  Use, duplication or disclosure restricted. 
 */

import java.io.Serializable;

import javax.xml.bind.annotation.XmlRootElement;

import com.fundi.framework.common.cslcomms.ConnectionServerDriver;
import com.fundi.framework.common.navigation.INavigationType;
import com.fundi.framework.common.nl1.Messages;
import com.fundi.framework.common.properties.ISelectionSource;
import com.fundi.framework.common.properties.PropertyDefn;

@XmlRootElement
public class ConnectionServer implements INavigationType, Serializable, ISelectionSource {
	
	//Eye catchers.
	public static final String EC_COPYRIGHT = "(C) Copyright 2011, 2016 Fundi Software.  Fundi Confidential."; //$NON-NLS-1$
	public static final String EC_APAR = "3.1.1.00_00014"; //$NON-NLS-1$
	public static final String EC_FMID = "H30SA10"; //$NON-NLS-1$
	public static final String EC_PID = "5655-CSL"; //$NON-NLS-1$
	
	private static final long serialVersionUID = 1L;

	private String name = ""; //$NON-NLS-1$
	private String host = ""; //$NON-NLS-1$
	private int port = 0;
	public static final String typeName = Messages.getString("ConnectionServer_6");  //$NON-NLS-1$
	private int connectStatus = AppInstance.CAN_CONNECT_UNKNOWN;
	private String connectionProfileName = ""; //$NON-NLS-1$
	private String[] availableProducts = null;
	
	/*
	 *  These variables aren't used yet.  They were added on 25/5/2013 in case we need to support SSL/TLS connections on a
	 *  Connection Server basis rather than Connection Profile basis.
	 */
	private boolean usingSSL = false;
	private String sslProtocolHandler = "com.sun.net.ssl.internal.www.protocol"; //$NON-NLS-1$
	private String keyStore = "C:\\Program Files\\Java\\jre\\lib\\security\\???.jks"; //$NON-NLS-1$
	private String keyStorePswd = "password"; //$NON-NLS-1$
	private String trustStore = "C:\\Program Files\\Java\\jre\\lib\\security\\???.jks"; //$NON-NLS-1$
	private String trustStorePswd = "password"; //$NON-NLS-1$
	
	private String connectFailureReason = "";
	
	public ConnectionServer() {
		
		super();
	}
	
	public ConnectionServer(String name, String host, int port) {

		setName(name);
		setHost(host);
		setPort(port);
	}
	
	public ConnectionServer(String[] resource) throws IllegalArgumentException {
		
		int index = resource.length - 1;
		String[] properties;
	
		if (index < 0) {
			throw new IllegalArgumentException(Messages.getString("ConnectionServer_7")); //$NON-NLS-1$
		} 
		else {
			if (resource[index].equals(getNavigationTypeName())) {
				//this is an empty instance of this object
			}
			else {
				index--;
				if (index >= 0 && resource[index].equals(getNavigationTypeName())) {
					properties = resource[index + 1].split(Messages.getString("ConnectionServer_8")); //$NON-NLS-1$
					setName(properties[0]);
					setHost(properties[1]);
					setPort(Integer.parseInt(properties[2]));
				}
				else {
					throw new IllegalArgumentException(Messages.getString("ConnectionServer_9")); //$NON-NLS-1$
				}
			}
		}
	}
	
	public int getConnectStatus() {
		return connectStatus;
	}
	
	public int testConnection(AppInstance instance) {
		
		setConnectStatus((new ConnectionServerDriver(instance, this)).testConnection());
		
		return getConnectStatus();
	}
	
	public Object testConnectionForIBMExplorer(AppInstance instance) {
		
		return (new ConnectionServerDriver(instance, this)).testConnection(true);
	}

	public void setConnectStatus(int connectStatus) {
		this.connectStatus = connectStatus;
	}
	
	public String getName() {
		
		return name;
	}

	public void setName(String name) {
		
		this.name = name;
	}

	public String getHost() {
		
		return host;
	}

	public void setHost(String host) {
		
		this.host = host;
	}

	public int getPort() {
		
		return port;
	}

	public void setPort(int port) {
		
		this.port = port;
	}
	
	public String getDisplayName() {
		
		return getName() + Messages.getString("ConnectionServer_10") + typeName + Messages.getString("ConnectionServer_11"); //$NON-NLS-1$ //$NON-NLS-2$
	}
	
	@Override
	public String getFullTitle() {

		return getDisplayName() + Messages.getString("ConnectionServer_12") + getHost() + Messages.getString("ConnectionServer_13") + getPort() + Messages.getString("ConnectionServer_14"); //$NON-NLS-1$ //$NON-NLS-2$ //$NON-NLS-3$
	}
	
	public void setConnectionProfileName(String name) {
		
		this.connectionProfileName = name;
	}
	
	public String getConnectionProfileName() {
		
		return connectionProfileName;
	}
	
	public String getConfigurationInfo(AppInstance instance) {
		
		return (new ConnectionServerDriver(instance, this)).getConfigurationInfo();
	}

	@Override
	public int hashCode() {
		final int prime = 31;
		int result = 1;
		result = prime * result + ((host == null) ? 0 : host.hashCode());
		result = prime * result + ((name == null) ? 0 : name.hashCode());
		result = prime * result + port;
		return result;
	}

	@Override
	public boolean equals(Object obj) {
		if (this == obj)
			return true;
		if (obj == null)
			return false;
		if (getClass() != obj.getClass())
			return false;
		ConnectionServer other = (ConnectionServer) obj;
		if (host == null) {
			if (other.host != null)
				return false;
		} else if (!host.equals(other.host))
			return false;
		if (name == null) {
			if (other.name != null)
				return false;
		} else if (!name.equals(other.name))
			return false;
		if (port != other.port)
			return false;
		return true;
	}

	@Override
	public String getNavigationTypeName() {

		return typeName;
	}

	@Override
	public PropertyDefn[] getPropertyDefns() {
		
		PropertyDefn[] defns = new PropertyDefn[4];
		
		defns[0] = new PropertyDefn(Messages.getString("ConnectionServer_1"), Messages.getString("ConnectionServer_1")); //$NON-NLS-1$ //$NON-NLS-2$
		defns[1] = new PropertyDefn(Messages.getString("ConnectionServer_3"), Messages.getString("ConnectionServer_3")); //$NON-NLS-1$ //$NON-NLS-2$
		defns[2] = new PropertyDefn(Messages.getString("ConnectionServer_5"), Messages.getString("ConnectionServer_5")); //$NON-NLS-1$ //$NON-NLS-2$
		defns[3] = new PropertyDefn(Messages.getString("ConnectionServer_0"), Messages.getString("ConnectionServer_0")); //$NON-NLS-1$ //$NON-NLS-2$

		return defns;
	}

	@Override
	
	public String getPropertyValue(String id) {
		
		if (id.equals(Messages.getString("ConnectionServer_1"))) { //$NON-NLS-1$
			return getName();
		}
		if (id.equals(Messages.getString("ConnectionServer_3"))) { //$NON-NLS-1$
			return getHost();
		}
		if (id.equals(Messages.getString("ConnectionServer_5"))) { //$NON-NLS-1$
			return Integer.toString(getPort());
		}
		if (id.equals(Messages.getString("ConnectionServer_0"))) { //$NON-NLS-1$
			return getConnectionProfileName();
		}
		
		return null;
	}

	public boolean isUsingSSL() {
		return usingSSL;
	}

	public void setUsingSSL(boolean usingSSL) {
		this.usingSSL = usingSSL;
	}

	public String getSslProtocolHandler() {
		return sslProtocolHandler;
	}

	public void setSslProtocolHandler(String sslProtocolHandler) {
		this.sslProtocolHandler = sslProtocolHandler;
	}

	public String getKeyStore() {
		return keyStore;
	}

	public void setKeyStore(String keyStore) {
		this.keyStore = keyStore;
	}

	public String getKeyStorePswd() {
		return keyStorePswd;
	}

	public void setKeyStorePswd(String keyStorePswd) {
		this.keyStorePswd = keyStorePswd;
	}

	public String getTrustStore() {
		return trustStore;
	}

	public void setTrustStore(String trustStore) {
		this.trustStore = trustStore;
	}

	public String getTrustStorePswd() {
		return trustStorePswd;
	}

	public void setTrustStorePswd(String trustStorePswd) {
		this.trustStorePswd = trustStorePswd;
	}

	public String getConnectFailureReason() {
		return connectFailureReason.trim();
	}

	public void setConnectFailureReason(String connectFailureReason) {
		this.connectFailureReason = connectFailureReason;
	}
}
